@@ -54,7 +54,7 @@ namespace eval ::tcl::tm {
     # Export the public API
 
     namespace export path
-    namespace ensemble create -command path -subcommand {add remove list}
+    namespace ensemble create -command path -subcommands {add remove list}
 }
 
 # ::tcl::tm::path implementations --
@@ -238,6 +238,15 @@ proc ::tcl::tm::UnknownHandler {original name args} {
 			continue
 		    }
 
+		    if {[package ifneeded $pkgname $pkgversion] ne {}} {
+			# There's already a provide script registered for
+			# this version of this package.  Since all units of
+			# code claiming to be the same version of the same
+			# package ought to be identical, just stick with
+			# the one we already have.
+			continue
+		    }
+
 		    # We have found a candidate, generate a "provide script"
 		    # for it, and remember it.  Note that we are using ::list
 		    # to do this; locally [list] means something else without
@@ -260,10 +269,8 @@ proc ::tcl::tm::UnknownHandler {original name args} {
 		    # Otherwise we still have to fallback to the regular
 		    # package search to complete the processing.
 
-		    if {
-			($pkgname eq $name) &&
-			[package vsatisfies $pkgversion {*}$args]
-		    } then {
+		    if {($pkgname eq $name)
+			    && [package vsatisfies $pkgversion {*}$args]} {
 			set satisfied 1
 
 			# We do not abort the loop, and keep adding provide
@@ -350,7 +357,7 @@ proc ::tcl::tm::Defaults {} {
 #	Calls 'path add' to paths to the list of module search paths.
 
 proc ::tcl::tm::roots {paths} {
-    foreach {major minor} [split [info tclversion] .] break
+    lassign [split [package present Tcl] .] major minor
     foreach pa $paths {
 	set p [file join $pa tcl$major]
 	for {set n $minor} {$n >= 0} {incr n -1} {
